#include "spm.h"
#include "spm_multiply.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>

void
spm_multiply_matrices(spm_matrix *m1, spm_matrix *m2, spm_matrix *m_out, const spm_multiply_info *minfo)
{
	int n_rows = m1->n_rows;
	int n_cols = m2->n_cols;
	
	assert(n_rows == m_out->n_rows);
	assert(n_cols == m_out->n_cols);
	assert(m1->n_cols == m2->n_rows);
	assert(minfo->in1 == m1->vinfo);
	assert(minfo->in2 == m2->vinfo);
	assert(minfo->out == m_out->vinfo);
	
	void *scratch1 = malloc(minfo->out->sizeof_value);
	void *scratch2 = malloc(minfo->out->sizeof_value);
	
	int row, col;
	for (row=0; row<n_rows; row++)
	{
		for (col=0; col<n_cols; col++)
		{
			int any_nonzero = 0;
			
			spm_matrix_element *e1, *e2;
			e1 = m1->rows[row];
			e2 = m2->cols[col];
			
			while(1)
			{
				if (e1->col == e2->row)
				{
					minfo->multiply_func(minfo->udata, SPM_M_VALUE(e1), SPM_M_VALUE(e2), scratch2);
					
					if (any_nonzero)
					{
						minfo->add_func(minfo->udata, scratch1, scratch2, scratch1);
					}
					else
					{
						memcpy(scratch1, scratch2, minfo->out->sizeof_value);
						any_nonzero = 1;
					}
					
					e1 = e1->next_in_row;
					e2 = e2->next_in_col;
				}
				
				while (e1 && e1->col < e2->row) e1 = e1->next_in_row;
				if (!e1) break;
				while (e2 && e2->row < e1->col) e2 = e2->next_in_col;
				if (!e2) break;
			}
			
			if (any_nonzero)
			{
				spm_matrix_set(m_out, row, col, scratch1);
			}
		}
	}
	
	free(scratch1);
	free(scratch2);
}

void
spm_multiply_matrices_to_densematrix(spm_matrix *m1, spm_matrix *m2, void *m_out, unsigned int stride, const spm_multiply_info *minfo)
{
	int n_rows = m1->n_rows;
	int n_cols = m2->n_cols;
	
	assert(stride >= n_cols * minfo->out->sizeof_value);
	assert(m1->n_cols == m2->n_rows);
	assert(minfo->in1 == m1->vinfo);
	assert(minfo->in2 == m2->vinfo);
	
	void *scratch = malloc(minfo->out->sizeof_value);
	
	int row, col;
	for (row=0; row<n_rows; row++)
	{
		for (col=0; col<n_cols; col++)
		{
			int any_nonzero = 0;
			
			spm_matrix_element *e1, *e2;
			e1 = m1->rows[row];
			e2 = m2->cols[col];
			
			void *dest_buf = m_out + stride*row + minfo->out->sizeof_value*col;
			
			while(1)
			{
				if (e1->col == e2->row)
				{
					minfo->multiply_func(minfo->udata, SPM_M_VALUE(e1), SPM_M_VALUE(e2), scratch);
					
					if (any_nonzero)
					{
						minfo->add_func(minfo->udata, dest_buf, scratch, dest_buf);
					}
					else
					{
						memcpy(dest_buf, scratch, minfo->out->sizeof_value);
						any_nonzero = 1;
					}
					
					e1 = e1->next_in_row;
					e2 = e2->next_in_col;
				}
				
				while (e1 && e1->col < e2->row) e1 = e1->next_in_row;
				if (!e1) break;
				while (e2 && e2->row < e1->col) e2 = e2->next_in_col;
				if (!e2) break;
			}
		}
	}
	
	free(scratch);
}