#ifndef SPM_UTILS
#define SPM_UTILS

/* spm_matrix_fix_row_pointers and spm_matrix_fix_col_pointers rebuild damaged row and column pointers in a matrix. spm_matrix_fix_row_pointers assumes that the column pointers are intact, although not necessarily in the correct order. spm_matrix_fix_col_pointers assumes the same for the row pointers. */
void
spm_matrix_fix_row_pointers(spm_matrix *m);
void
spm_matrix_fix_col_pointers(spm_matrix *m);

/* spm_matrix_find_or_create_element looks for an element at (row,col). If it finds it, then it sets *e to the element found and returns 1. Otherwise, it creates a new element at (row,col), sets *e to the new element, and returns 0. The new element is not initialized, so the caller must set its value. */
int
spm_matrix_find_or_create_element(spm_matrix *m, int row, int col, spm_matrix_element **e);

/* spm_vector_find_or_create_element looks for an element at 'index'. If it finds it, then it sets *e to the element found and returns 1. Otherwise, it creates a new element at 'index', sets *e to the new element, and returns 0. The new element is not initialized, so the caller must set its value. */
int
spm_vector_find_or_create_element(spm_vector *v, int index, spm_vector_element **e);

/* spm_matrix_resize changes the size of a matrix. If the new size is larger than the old size, then the new elements will be zero. If the new size is smaller than the old size, then any elements outside the new size will be removed. */
void
spm_matrix_resize(spm_matrix *m, int n_rows, int n_cols);

/* spm_matrix_transfer_cols transfers a group of columns from one matrix to another. The destination matrix must be empty. All columns such that start <= colnumber < start+count will be transferred. If start!=dstart, then the columns will be renumbered such that the first column is numbered dstart. The two matrices must have the same number of rows.

THIS FUNCTION IS CURRENTLY BROKEN. */
void
spm_matrix_transfer_cols(spm_matrix *src, int start, int count, spm_matrix *dest, int dstart);

/* spm_matrix_del_row and spm_matrix_del_col remove the specified row or column from the given matrix. The height or width of the matrix will be reduced by 1. */
void
spm_matrix_del_row(spm_matrix *m, int row);

void
spm_matrix_del_col(spm_matrix *m, int col);

/* spm_matrix_print_pointers is a debugging function. It prints the matrix, but instead of printing the values it prints the address of each spm_matrix_element. */
void
spm_matrix_print_pointers(spm_matrix *m);

#endif