#ifndef SPM_VAlUE_DOUBLE_H
#define SPM_M_VALUE_DOUBLE_H

#include <stdio.h>

#include "spm.h"
#include "spm_multiply.h"
#include "spm_gaussian.h"

/* Many users of SPM will probably want to use double-precision floating point numbers as their data type. To make this easier, a set of simple routines and info-structs has been provided for working with doubles. These functions and info-structures can be used to easily work with sparse matrices of doubles. */

/* A value-info structure describing doubles */
extern const spm_value_info
spm_double_value_info;

/* A structure suitable for use with spm_multiply_matrices */
extern const spm_multiply_info
spm_double_multiply_info;

/* A structure suitable for use with spm_gaussian or spm_gaussian_vector */
extern const spm_gaussian_info
spm_double_gaussian_info;

/* A structure suitable for use with spm_gaussian_vector */
extern const spm_gaussianaugmentation_info
spm_double_gaussianaugmentation_info;

/* A function that allocates a new sparse matrix and automatically gives it spm_double_value_info as its value-info */
spm_matrix *
spm_matrix_double_alloc(int n_rows, int n_cols);

/* A convenient function to set an element of a double-matrix */
void
spm_matrix_double_set(spm_matrix *m, int row, int col, double value);

/* A convenient function to get an element of a double-matrix */
double
spm_matrix_double_get(spm_matrix *m, int row, int col);

/* spm_matrix_double_write writes a human-readable representation of a sparse matrix to a file. It only uses the row pointers, so it can be used while debugging a function that breaks column pointers. */
void
spm_matrix_double_write(spm_matrix *m, FILE *f);

/* spm_matrix_double_print(m) is spm_matrix_double_write(m, stdout). */
void
spm_matrix_double_print(spm_matrix *m);

#endif