#include "spm.h"
#include "spm_utils.h"
#include <assert.h>
#include <stdlib.h>
#include <string.h>

spm_vector *
spm_vector_alloc(int size, const spm_value_info *vinfo)
{
	spm_vector *v = malloc(sizeof(spm_vector));
	if (!v)
	{
		return NULL;
	}
	
	v->size = size;
	
	v->zero_value = malloc(vinfo->sizeof_value);
	if (!v->zero_value)
	{
		free(v);
		return NULL;
	}
	vinfo->set_zero_func(vinfo->udata, v->zero_value);
	
	v->vinfo = vinfo;
	
	v->first = NULL;
	
	return v;
}

void
spm_vector_free(spm_vector *v)
{
	spm_vector_element *e, *next;
	for (e=v->first; e; e=next)
	{
		next = e->next;
		if (v->vinfo->free_func)
		{
			v->vinfo->free_func(v->vinfo->udata, SPM_V_VALUE(e));
		}
		free(e);
	}
	
	if (v->vinfo->free_func)
	{
		v->vinfo->free_func(v->vinfo->udata, v->zero_value);
	}
	free(v->zero_value);
	
	free(v);
}

void
spm_vector_set_nonzero(spm_vector *v, int n, const void *data)
{
	int already_existed;
	spm_vector_element *e;
	already_existed = spm_vector_find_or_create_element(v, n, &e);
	
	if (already_existed)
	{
		if (v->vinfo->free_func)
		{
			v->vinfo->free_func(v->vinfo->udata, SPM_V_VALUE(e));
		}
	}
	memcpy(SPM_V_VALUE(e), data, v->vinfo->sizeof_value);
}

void
spm_vector_set_zero(spm_vector *v, int n)
{
	spm_vector_element **slot;
	for (slot=&v->first; *slot && (*slot)->index<n; slot=&(*slot)->next)
	{
		continue;
	}
	
	if (!*slot || (*slot)->index>n)
	{
		/* Nothing to be done */
		return;
	}
	else
	{
		spm_vector_element *to_free = *slot;
		*slot = to_free->next;
		
		if (v->vinfo->free_func)
		{
			v->vinfo->free_func(v->vinfo->udata, SPM_V_VALUE(to_free));
		}
		
		free(to_free);
	}
}

void
spm_vector_set(spm_vector *v, int n, const void *data)
{
	assert(v);
	assert(n < v->size);
	assert(data);
	
	if (v->vinfo->is_zero_func(v->vinfo->udata, data))
	{
		spm_vector_set_zero(v, n);
	}
	else
	{
		spm_vector_set_nonzero(v, n, data);
	}
}

void
spm_vector_get(spm_vector *v, int n, void *data)
{
	spm_vector_element *e;
	for (e=v->first; e; e=e->next)
	{
		if (e->index == n)
		{
			memcpy(data, SPM_V_VALUE(e), v->vinfo->sizeof_value);
			return;
		}
	}
	
	memcpy(data, v->zero_value, v->vinfo->sizeof_value);
}