#include "spm.h"
#include "spm_value_double.h"
#include "spm_gaussian.h"
#include <assert.h>
#include <math.h>

#define EQUAL(a,b) (fabs((a)-(b))<0.0000001)

int main(int argc, char *argv[])
{
	/* This test solves the following system of linear equations:
	
	5X + Y = 7
	5X - Y = 3
	
	*/
	
	spm_matrix *eqs = spm_matrix_double_alloc(2,2);
	
	spm_matrix_double_set(eqs, 0, 0, 5);
	spm_matrix_double_set(eqs, 0, 1, 1);
	spm_matrix_double_set(eqs, 1, 0, 5);
	spm_matrix_double_set(eqs, 1, 1, -1);
	
	double constants[2] = {7, 3};
	
	int ok = spm_gaussian_densevector(eqs, &spm_double_gaussian_info,
							          constants, &spm_double_gaussianaugmentation_info);
	assert(ok==0);
	
	/* The solution is:
	
	X = 1
	Y = 2
	
	*/
	
	assert(EQUAL(constants[0], 1));
	assert(EQUAL(constants[1], 2));
	
	spm_matrix_free(eqs);
	
	return 0;
}