#include "spm.h"
#include "spm_value_double.h"
#include "spm_gaussian.h"
#include <assert.h>
#include <math.h>

#define EQUAL(a,b) (fabs((a)-(b))<0.0000001)

int main(int argc, char *argv[])
{
	/* This test solves the following system of linear equations:
	
	10X +  Y = A
	 5X - 2Y = B
	
	It finds X and Y in terms of A and B. It is the same thing as inverting the following matrix:
	
	[5  1]
	[5 -1]
	
	*/
	
	spm_matrix *eqs = spm_matrix_double_alloc(2,2);
	
	spm_matrix_double_set(eqs, 0, 0, 10);
	spm_matrix_double_set(eqs, 0, 1,  1);
	spm_matrix_double_set(eqs, 1, 0,  5);
	spm_matrix_double_set(eqs, 1, 1, -2);
	
	/* The augmentation should initially be an identity matrix. When finished, it will be the solution to the system. */
	
	spm_matrix *result = spm_matrix_double_alloc(2,2);
	spm_matrix_identity(result);
	
	int ok = spm_gaussian_matrix(eqs, &spm_double_gaussian_info,
							     result, &spm_double_gaussianaugmentation_info);
	assert(ok==0);
	
	/* The solution is:
	
	X = 0.08*A + 0.04*B
	Y = 0.20*A - 0.40*B
	
	*/
	
	assert(EQUAL(spm_matrix_double_get(result,0,0),  0.08));
	assert(EQUAL(spm_matrix_double_get(result,0,1),  0.04));
	assert(EQUAL(spm_matrix_double_get(result,1,0),  0.20));
	assert(EQUAL(spm_matrix_double_get(result,1,1), -0.40));
	
	spm_matrix_free(eqs);
	spm_matrix_free(result);
	
	return 0;
}
