#include "spm.h"
#include "spm_value_double.h"
#include "spm_gaussian.h"
#include <assert.h>
#include <math.h>

#define EQUAL(a,b) (fabs((a)-(b))<0.0000001)

int unknown_cb(void *udata, unsigned int col, void *value);

int main(int argc, char *argv[])
{
	/* This test solves the following equation:
	
	X + Y = 9
	
	Obviously, this system of equations is underspecified - there are many possible values for X and
	Y. The solver will arbitrarily pick one and use it.
	
	*/
	
	spm_matrix *eqs = spm_matrix_double_alloc(2,2);
	
	spm_matrix_double_set(eqs, 0, 0, 1);
	spm_matrix_double_set(eqs, 0, 1, 1);
	
	/* Put a second row into the matrix, but make it empty. */
	spm_matrix_double_set(eqs, 1, 0, 0);
	spm_matrix_double_set(eqs, 1, 1, 0);
	
	double constants[2] = {9, 0};
	
	int ok = spm_gaussian_densevector_underspecified(eqs,
		&spm_double_gaussian_info,
		constants,
		&spm_double_gaussianaugmentation_info,
		unknown_cb,
		(void*)1);
	assert(ok==0);
	
	/* The solution is either:
	
	X = 5
	Y = 4
	
	or:
	
	X = 4
	Y = 5
	
	*/
	
	assert((EQUAL(constants[0], 4) && EQUAL(constants[1],5)) ||
		   (EQUAL(constants[0], 5) && EQUAL(constants[1],4)));
	
	spm_matrix_free(eqs);
	
	return 0;
}

int unknown_cb(void *udata, unsigned int col, void *value)
{
	assert((int)udata == 1);
	
	/* Arbitrarily use the value of 5 */
	*(double*)value = 5;
	
	return 1; /* Tell the Gauss-Jordan solver it is okay to continue */
}