#include "spm.h"
#include "spm_value_double.h"
#include "spm_utils.h"
#include <assert.h>
#include <math.h>

#define EQUAL(a,b) (fabs((a)-(b))<0.0000001)

int main(int argc, char *argv[])
{
	int x,y;
	spm_matrix *m;
	
	/* This test cuts off the right column of the following matrix:
	
	[1 2 3]
	[4 5 6]
	
	while simultaneously adding an extra row.
	*/
	
	m = spm_matrix_double_alloc(2,3);
	
	for (x=0; x<3; x++)
	{
		for (y=0; y<2; y++)
		{
			spm_matrix_double_set(m, y, x, 1+x+3*y);
		}
	}
	
	spm_matrix_resize(m, 3, 2);
	
	assert(m->n_rows==3);
	assert(m->n_cols==2);
	assert(EQUAL(spm_matrix_double_get(m,2,0), 0));
	
	spm_matrix_free(m);
	
	/* This tests deletes the second row and second column from the following matrix:
	
	[1 2 3]
	[4 5 6]
	[7 8 9]
	
	*/
	
	m = spm_matrix_double_alloc(3,3);
	
	for (x=0; x<3; x++)
	{
		for (y=0; y<3; y++)
		{
			spm_matrix_double_set(m, y, x, 1+x+3*y);
		}
	}
	
	spm_matrix_del_row(m, 1);
	spm_matrix_del_col(m, 1);
	
	assert(m->n_rows==2);
	assert(m->n_cols==2);
	assert(spm_matrix_double_get(m, 1, 1)==9);
	assert(spm_matrix_double_get(m, 1, 0)==7);
	
	return 0;
}